
/*
 * $Id: AsyncClt.cc,v 1.5 1999/09/07 17:45:59 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "LogComment.h"
#include "RobotSym.h"
#include "ErrorMgr.h"
#include "AsyncClt.h"

#include "rndDistrs.h"
#include "polyErrors.h"

void AsyncClt::configure(const RobotSym *cfg) {
	Client::configure(cfg);

	// poisson request stream is modeled using an exponential
	// distribution with a mean of a given request rate
	double rr;
	static RndGen rng; // all clients should share one rng
	Assert(cfg->reqRate(rr));

	if (rr <= 0) {
		Comment << "request rate for robot " << theKind << " is not positive" << endl;
		FatalError(errOther);
	}

	Assert(!theThinkDistr);
	theThinkDistr = new ExpDistr(&rng, 1/rr);
}

void AsyncClt::wakeUp(const Alarm &a) {
	Client::wakeUp(a);
	scheduleLaunch(a.time()); // next
	launch();
}
