
/*
 * $Id: CltConnMgr.cc,v 1.8 1999/09/07 17:46:01 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "CltConnMgr.h"


CltConnMgr::CltConnMgr(): theIdleLvlLmt(-1) {
}

CltConnMgr::~CltConnMgr() {
	while (theIdleConns.count())
		closeIdle(theIdleConns.dequeue(), ConnCloseStat::ckActive);
}

void CltConnMgr::configure(const NetAddr &anAddr, const NetAddr &anOrgAddr, const SockOpt &anOpt, int anIdleLvlLmt, RndDistr *aPconnUseLmt) {
	ConnMgr::configure(anAddr, anOrgAddr, anOpt, aPconnUseLmt);
	theIdleLvlLmt = anIdleLvlLmt;
}

Connection *CltConnMgr::get() {
	Connection *conn = 0;
	if (theIdleConns.count()) {
		conn = theIdleConns.dequeue();
		conn->theRd.stop(this);
	} else {
		conn = open();
	}

	if (conn)
		conn->use();
	return conn;
}

Connection *CltConnMgr::open() {
	Connection *conn = TheConnFarm.get();
	if (conn->connect(theAddr, theSockOpt)) {
		opened(conn);
		return conn;
	} else {
		// connect failed
		TheConnFarm.put(conn);
		return 0;
	}
}

// called on an idle connection closed by the other side
void CltConnMgr::noteReadReady(int fd) {
	Connection *conn = TheConnIdx[fd];
	Assert(conn);

	// delete connection from the idle queue
	theIdleConns.dequeue(conn);
	closeIdle(conn, ConnCloseStat::ckIdleForeign);
}

void CltConnMgr::putIdle(Connection *conn) {
	// first observe imposed limit on idle connections (if any)

	// we are not allowed to keep anything
	if (theIdleLvlLmt == 0) { 
		close(conn);
		return;
	}

	// we are allowed, but out of space
	if (theIdleLvlLmt > 0 && theIdleConns.count() >= theIdleLvlLmt) {
		// close the oldest connection, get space for the fresh one
		closeIdle(theIdleConns.dequeue(), ConnCloseStat::ckIdleLocal);
	}

	conn->theRd.start(this); // watch for close on the other end
	theIdleConns.enqueue(conn);
}

void CltConnMgr::closeIdle(Connection *conn, ConnCloseStat::CloseKind ck) {
	Assert(conn);
	conn->theRd.stop(this);
	conn->closeKind(ck); 
	close(conn);
}
