
/*
 * $Id: ErrorMgr.cc,v 1.8 1999/09/07 17:46:07 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "ErrorStrTbl.h"
#include "LogComment.h"
#include "ErrorMgr.h"
#include "polyBcastChannels.h"

ErrorMgr TheErrorMgr;


ErrorMgr::ErrorMgr(): theVerbLvl(1), theStrs(0), theTotCnt(0) {
}

void ErrorMgr::configure(const ErrorStrTbl *aStrs) {
	Assert(aStrs);
	Assert(!theStrs);
	theStrs = aStrs;

	theCnts.maxErrNo(aStrs->maxErrNo());
	theExplDlines.stretch(theCnts.count());
	theExplDlines.count(theExplDlines.capacity());
}

void ErrorMgr::canonError(int &e) const {
	if (e < 0 || e > theCnts.maxErrNo())
		e = theCnts.maxErrNo();
}

bool ErrorMgr::needExpl(int e) {
	canonError(e);
	if (theVerbLvl > 0 && theCnts[e] >= theExplDlines[e]) {
		theExplDlines[e] += 1 + theExplDlines[e]/theVerbLvl;
		return true;
	}
	return false;
}

void ErrorMgr::noteError(int e) {
	canonError(e);
	theCnts.record(e);
	theTotCnt++;
	Broadcast(TheErrChannel, e);
}

void ErrorMgr::reportError(const SrcLoc &loc, int e) {
	canonError(e);
	Comment(0)
		<< loc << "error: "
		<< theCnts[e] << '/' << theTotCnt
		<< " (" << e << ") "
		<< (*theStrs)[e]
		<< endc;
}

const ErrorStrTbl *ErrorMgr::strings() const {
	return theStrs;
}

const ErrorStat &ErrorMgr::stats() const {
	return theCnts;
}


/* global routines */

void NoteError(int e) {
	TheErrorMgr.noteError(e);
}

bool ExplainError(int e) {
	TheErrorMgr.noteError(e);
	return TheErrorMgr.needExpl(e);
}

void ReportErrorLoc(const SrcLoc &loc, int e) {
	TheErrorMgr.noteError(e);
	if (TheErrorMgr.needExpl(e))
		TheErrorMgr.reportError(loc, e);
}

void FatalErrorLoc(const SrcLoc &loc, int e) {
	TheErrorMgr.noteError(e);
	TheErrorMgr.reportError(loc, e);
	Comment(0) << "last error (" << e << ") was fatal; terminating..." << endc;
	exit(e);
}
