
/*
 * $Id: LevelStat.cc,v 1.2 1999/09/07 17:46:14 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "Clock.h"
#include "ILog.h"
#include "OLog.h"
#include "LevelStat.h"
#include "gadgets.h"

LevelStat::LevelStat() {
	theIncCnt = theDecCnt = theLevel = 0;
	theSum = Time(0, 0);
}

void LevelStat::restart() {
	theIncCnt = theDecCnt = 0; // but keep level
	theStart = theCurStart = TheClock;
	theSum = Time(0, 0);
}

void LevelStat::change() {
	if (theStart < 0)
		theStart = TheClock;
	else
		theSum += (TheClock - theCurStart) * theLevel;
	theCurStart = TheClock;
}

double LevelStat::mean() const {
	return theCurStart > theStart ?
		theSum/(theCurStart-theStart) : 
		-1; 
}

void LevelStat::store(OLog &log) const {
	log << theIncCnt << theDecCnt << theLevel
		<< theStart << theCurStart << theSum;
}

void LevelStat::load(ILog &log) {
	log >> theIncCnt >> theDecCnt >> theLevel
		>> theStart >> theCurStart >> theSum;
}

void LevelStat::mergeWith(const LevelStat &s) {
	if (s.theStart > 0) {
		theIncCnt += s.theIncCnt;
		theDecCnt += s.theDecCnt;
		theLevel += s.theLevel;

		// note: this will produce correct means for levels
		// with different start/stop times, but usually
		// merged_mean will not be (mean1 + mean2)/2.
		theStart = Min(theStart, s.theStart);
		// assume that the curStart is also the end
		theCurStart = Max(theCurStart, s.theCurStart);
		theSum += s.theSum;
	}
}
