
/*
 * $Id: PolyClt.cc,v 1.7 1999/09/07 17:46:21 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "AsyncClt.h"
#include "SyncClt.h"
#include "EphPortMgr.h"
#include "ExpPortMgr.h"
#include "OLog.h"
#include "LogComment.h"
#include "SharedOpts.h"
#include "CltOpts.h"
#include "CltConnMgr.h"
#include "PopModel.h"
#include "TmlModel.h"
#include "HitModel.h"
#include "World.h"
#include "RobotSym.h"
#include "PolyClt.h"

#include "gadgets.h"
#include "rndDistrs.h"
#include "globals.h"
#include "shutdown.h"

Agent *PolyClt::makeAgent(const AgentSym *c) {
	Assert(c);

	// filter out robots
	if (!c->isA("Robot"))
		return 0;
	const RobotSym &cfg = (const RobotSym&)c->cast("Robot");

	double rr;
	Client *client = 0;
	if (cfg.reqRate(rr)) {
		client = new AsyncClt();
	} else {
		client = new SyncClt();
	}

	Assert(client);
	client->configure(&cfg);

	return client;
}

void PolyClt::configure() {

	PolyApp::configure();

	// configure the World
	TheWorld.type(TheCltOpts.theWorld.theType);
	TheWorld.id(TheCltOpts.theWorld.theId);
	TheWorld.theHitMdl->hr(TheCltOpts.theWorld.theHitRatio);
	TheWorld.thePopMdl = TheCltOpts.theWorld.thePopMdl.model();
	if (TheCltOpts.theWorld.theTmlDistr) {
		TheWorld.theTmlMdl->distDistr(TheCltOpts.theWorld.theTmlDistr.condDistr());
		TheWorld.theTmlMdl->searchDelta(TheCltOpts.theWorld.theTmlDelta);
		TheWorld.theTmlMdl->planDepth(TheCltOpts.theWorld.theTmlDepth);
	}

	if (TheCltOpts.theWorld.theUrlsFileName &&
		!TheWorld.loadUrls(TheCltOpts.theWorld.theUrlsFileName)) {
		cerr << ThePrgName << ": failed to load urls from `" << TheCltOpts.theWorld.theUrlsFileName << "'" << endl << endc;
	}

	// explicit or ephemeral binding
	ThePortMgr = TheCltOpts.thePorts.set() ?
		(PortMgr*) new ExpPortMgr(TheCltOpts.thePorts.lo(), TheCltOpts.thePorts.hi()) :
		(PortMgr*) new EphPortMgr();

	if (TheCltOpts.theWorld.thePrefilledCnt > 0)
		TheWorld.prefill(TheCltOpts.theWorld.thePrefilledCnt);
}

void PolyClt::getOpts(Array<OptGrp*> &opts) {
	PolyApp::getOpts(opts);
	opts.append(&TheCltOpts);
}

void PolyClt::reportCfg() {
	PolyApp::reportCfg();
}



int main(int argc, char *argv[]) {
	PolyClt app;
	return app.run(argc, argv);
}
