
/*
 * $Id: PolyOLog.cc,v 1.4 1999/09/07 17:46:21 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "Clock.h"
#include "Progress.h"
#include "PolyOLog.h"
#include "polyLogTags.h"

PolyOLog TheOLog; // assume no static-ctors will log

PolyOLog::PolyOLog() {
	// stretch to the maximum
	theDir.put(0, lgEnd);
}

void PolyOLog::period(Time aPeriod) {
	Assert(aPeriod > 0);
	Assert(theFD >= 0);
	thePeriod = aPeriod;
	sleepFor(thePeriod);
}

void PolyOLog::wakeUp(const Alarm &alarm) {
	const Size pageSize = 4096; // XXX: typical page size?
	AlarmUser::wakeUp(alarm);
	if (theFD >= 0) {
		// flush at most two pages at a time
		if (size() >= pageSize)
			flush(size() >= pageSize*2 ? pageSize*2 : pageSize);
		
		sleepTill(alarm.time() + thePeriod);
	}
	// do not set alarms if closed
}

void PolyOLog::putHeader() {
	puti(lgMagic1); puti(lgMagic2); puti(0); // magic
	OLog::putHeader();
}

void PolyOLog::putTrailer() {
	OLog::putTrailer();
	puti(0); puti(lgMagic2); puti(lgMagic1); // magic
}

void PolyOLog::begEntry(int tag) {

	if (theModTime != TheClock) {
		theModTime = TheClock;
		OLog::begEntry(lgProgress);
		TheProgress.store(*this);
		endEntry();
	}

	OLog::begEntry(tag);
}
