
/*
 * $Id: UniqId.cc,v 1.2 1999/09/07 17:46:33 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include <unistd.h>
#include <iostream.h>
#include <strstream.h>

#include "ILog.h"
#include "OLog.h"
#include "UniqId.h"
#include "gadgets.h"

static UniqId TheSessionId;

static void refreshSessionId();



UniqId::UniqId(): theSecs(0), theMix(0), theCnt(0) {
}

UniqId::UniqId(int aSecs, int aMix, int aCnt): theSecs(aSecs), theMix(aMix), theCnt(aCnt) {
	Assert(aCnt <= 0 || (aSecs > 0 && aMix > 0));
}

UniqId &UniqId::create() {
	if (TheSessionId.theCnt <= 0)
		refreshSessionId();
	*this = TheSessionId;
	TheSessionId.theCnt += 2;
	return *this;
}

ostream &UniqId::print(ostream &os) const {
	return os << theSecs << '.' << theMix << ':' << theCnt;
}

OLog &UniqId::store(OLog &ol) const {
	return ol << theSecs << theMix << theCnt;
}

ILog &UniqId::load(ILog &il) {
	return il >> theSecs >> theMix >> theCnt;
}

String Id2Str(const UniqId &id) {
	char buf[64];
	ostrstream os(buf, sizeof(buf));
	id.print(os) << ends;
	return buf;
}

// it would be better to use hostid in a mix, but gethostid() is not portable
// so we use tm.usec instead
static
void refreshSessionId() {
	static unsigned pid = (unsigned) getpid();
	Time tm(Time::Now());

	TheSessionId = UniqId(
		0x7fffffff & (int)(tm.sec() - 912345678),
		0x7fffffff & (int)((tm.usec() << 16) | (pid & 0xffff)),
		2);
}
