
/*
 * $Id: WorldOpts.cc,v 1.9 1999/09/07 17:46:34 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "UniqId.h"
#include "Rnd.h"
#include "ZipfPopMdl.h"
#include "UnifPopMdl.h"
#include "WorldOpts.h"


WorldOpts::WorldOpts():
	theType(this,      "world_type <str>",   "part of a URL", "Polygraph"),
	theId(this,        "world_id <str>",     "part of a URL", Id2Str(UniqId::Create())),
	theUrlsFileName(this,"world_urls <file>","file with URLs"),

	theUniqStart(this, "unique_urls <int>",  "append unique suffix (counter) to URLs"),
	theReqOrder(0,     "order <int>",        "request order"),

	theHitRatio(this,  "recur <ratio>",      "recurrence (prob of revisiting a URL)"),
	thePopMdl(this,    "pop_model <model>",  "popularity model", new UnifPopMdl()),
	theTmlDistr(this,  "tmp_loc <distr>",    "distances distr for temporal locality"),
	theTmlDelta(this,  "tmp_loc_delta <int>","fuzziness of temporal locality choices", 1),
	theTmlDepth(this,  "tmp_loc_depth <int>","max depth of tmp locality plan", 500000),

	theExpOidCnt(0,    "exp_oids <int>",     "how many unique oids to expect"),
	thePrefilledCnt(this,"prefilled_cnt <int>","number of already cached oids")

{
	theTmlDistr.argType("num");
}

bool WorldOpts::validate() const {
	if (!thePopMdl)
		cerr << "popularity model must be set" << endl;
	else
	if (theHitRatio > 1)
		cerr << "document hit ratio cannot be higher than 1, got: " << theHitRatio << endl;
	else
		return OptGrp::validate();
	return false;
}


/* PopMdlOpt */

PopMdlOpt::PopMdlOpt(OptGrp *aGrp, const char *aName, const char *aDescr, PopModel *def):
	Opt(aGrp, aName, aDescr), theVal(def) {
}

bool PopMdlOpt::parse(const char *name, const char *val) {
	if (val) {
		delete theVal;
		theVal = 0;

		if (!strcmp(val, "zipf")) {
			theVal = new ZipfPopMdl();
		} else
		if (!strcmp(val, "unif")) {
			theVal = new UnifPopMdl();
		}

		if (theVal)
			return true;
	}
	cerr << "popularity model (e.g., `zipf') expected for the `" << name << "` option; got `" << valStr(val) << "'" << endl;
	return false;
}

void PopMdlOpt::report(ostream &os) const {
	if (theVal)
		os << *theVal;
	else
		os << "<none>";
}
