
/*
 * $Id: Xaction.cc,v 1.26 1999/09/07 17:46:35 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "Connection.h"
#include "Clock.h"
#include "Progress.h"
#include "ErrorMgr.h"
#include "SharedOpts.h"
#include "Xaction.h"

#include "polyBcastChannels.h"
#include "polyErrors.h"
#include "globals.h"


void Xaction::reset() {
	theConn = 0;
	theId.clear();
	theStartTime = theLifeTime = Time();
	theRepSize = -1;
	theOidRec.reset();
	theState = stNone;
}

void Xaction::start(Connection *aConn) {
	Assert(aConn);
	theConn = aConn;

	theStartTime = theConn->useStart();
	Broadcast(TheXactBegChannel, this);
}

void Xaction::stop(int errNo) {
	Assert(theConn);
	if (theState == stSpaceWaiting)
		theConn->theWr.stop(this);
	else
		theConn->theRd.stop(this);
	
	theLifeTime = TheClock - theStartTime;

	if (!errNo) {
		stopStats();
	} else {
		theConn->bad(true);
		// low level errors should be reported when they occur
		// low level errors are propogated to us as "-1"
		// do not count/report them twice
		if (errNo > 0)
			ReportError(errNo);
		Broadcast(TheXactErrChannel, this);
		TheProgress.failure();
	}

	theState = stDone;
}

void Xaction::stopStats() {
	TheProgress.success();
	Broadcast(TheXactEndChannel, this);
}
