
/*
 * $Id: ErrorMgr.h,v 1.8 1999/09/07 17:46:54 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_ERROR_MANAGER_H
#define PG_ERROR_MANAGER_H

#include "Array.h"
#include "SrcLoc.h"
#include "ErrorStat.h"

class ErrorStrTbl;

// central point for all error events
class ErrorMgr {
	public:
		ErrorMgr();

		void configure(const ErrorStrTbl *aStrs);

		void noteError(int e);
		void reportError(const SrcLoc &loc, int e);
		bool needExpl(int e);

		const ErrorStrTbl *strings() const;
		const ErrorStat &stats() const;

	protected:
		void canonError(int &e) const;

	protected:
		int theVerbLvl;             // how often to explain errors
		const ErrorStrTbl *theStrs; // errno strings

		ErrorStat theCnts;          // error counters
		Array<int> theExplDlines;   // time of next report
		int theTotCnt;              // total number of errors
};

extern ErrorMgr TheErrorMgr;

extern void NoteError(int e);    // just silently count it
extern bool ExplainError(int e); // count and return true if should explain
                                 
extern void ReportErrorLoc(const SrcLoc &, int e);  // count and maybe print a std explanation
extern void FatalErrorLoc(const SrcLoc &, int e);   // count, report, exit

// macros to auto supply source location
#define ReportError(e) ReportErrorLoc(Here, (e))
#define FatalError(e) FatalErrorLoc(Here, (e))

#endif
