
/*
 * $Id: HRStat.h,v 1.4 1999/09/07 17:46:58 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_HIT_RATIOS_STATS_H
#define PG_HIT_RATIOS_STATS_H

#include "AggrStat.h"
#include "CntSzStat.h"

// handy for keeping track of hit ratios, cachability ratios, etc.
// "hit" and "miss" classes must be disjoint
class HRStat {
	public:
		HRStat();

		void reset();
		OLog &store(OLog &log) const;
		ILog &load(ILog &);

		void record(Size sz, bool hit) { hit ? theHits.record(sz): theMisses.record(sz); }

		int hits() const { return theHits.count(); }
		int misses() const { return theMisses.count(); }
		int xacts() const { return theHits.count() + theMisses.count(); }

		BigSize xactSz() const { return theHits.size() + theMisses.size(); }

		double dhr() const;
		double bhr() const;
		double dhp() const;
		double bhp() const;

		HRStat &operator +=(const HRStat &s);

	protected:
		CntSzStat theHits;
		CntSzStat theMisses;
};

inline OLog &operator <<(OLog &ol, const HRStat &s) { return s.store(ol); }
inline ILog &operator >>(ILog &il, HRStat &s) { return s.load(il); }

#endif
