
/*
 * $Id: IntvlLx.h,v 1.5 1999/09/07 17:47:02 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_STAT_INTERVAL_LOG_EXTRACTOR_H
#define PG_STAT_INTERVAL_LOG_EXTRACTOR_H

#include "LogExtractor.h"
#include "StatIntvlRec.h"

class IntvlLxRec: public LxRec {
	public:
		virtual const StatIntvlRec &intvl() const { return theRec; }

	protected:
		virtual const char *type() const { return "intvl"; }

	public:
		StatIntvlRec theRec;
};

// extracts one object from an entry
class IntvlLx: public LogExtractor {
	private:
		typedef void (IntvlLx::*Exor)(const StatIntvlRec &i, ostream &os) const;

	public:
		IntvlLx();

		virtual bool configure(const String &aName, const LxParams &params);
		virtual void extract(const LxRec &r, ostream &os);

	protected:
		void exReqRate(const StatIntvlRec &i, ostream &os) const { os << i.reqRate(); }
		void exRepRate(const StatIntvlRec &i, ostream &os) const { os << i.repRate(); }

		void exConnTtlMean(const StatIntvlRec &i, ostream &os) const { os << i.theConnLifeTm.mean(); }
		void exConnUseMean(const StatIntvlRec &i, ostream &os) const { os << i.theConnUseCnt.mean(); }

		void exReqDHR(const StatIntvlRec &i, ostream &os) const { os << i.theIdealHR.dhp(); }
		void exReqBHR(const StatIntvlRec &i, ostream &os) const { os << i.theIdealHR.bhp(); }
		void exRepDHR(const StatIntvlRec &i, ostream &os) const { os << i.theRealHR.dhp(); }
		void exRepBHR(const StatIntvlRec &i, ostream &os) const { os << i.theRealHR.bhp(); }

		void exRepDCR(const StatIntvlRec &i, ostream &os) const { os << i.theChbR.dhp(); }
		void exRepBCR(const StatIntvlRec &i, ostream &os) const { os << i.theChbR.bhp(); }

		void exAllRptmMean(const StatIntvlRec &i, ostream &os) const { os << i.respTime().mean(); }
		void exHitRptmMean(const StatIntvlRec &i, ostream &os) const { os << i.theHitRespTm.mean(); }
		void exMissRptmMean(const StatIntvlRec &i, ostream &os) const { os << i.theMissRespTm.mean(); }

		void exFillSz(const StatIntvlRec &i, ostream &os) const { os << i.theFill.size().byted(); }
		void exFillCnt(const StatIntvlRec &i, ostream &os) const { os << i.theFill.count(); }

		void exErrCnt(const StatIntvlRec &i, ostream &os) const { os << i.theErrCnt; }
		void exErrPcnt(const StatIntvlRec &i, ostream &os) const { os << i.errPercent(); }

		void exDuration(const StatIntvlRec &i, ostream &os) const { os << i.theDuration.secd(); }

	private:
		Exor theExor; // set at configure time
};

#endif
