
/*
 * $Id: Opt.h,v 1.5 1999/09/07 17:47:11 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_OPT_CLASS_H
#define PG_OPT_CLASS_H

#include "String.h"
#include "Array.h"


class Opt;
class CmdLine;

class OptGrp: public Array<Opt*> {
	public:
		virtual ~OptGrp() {}

		virtual ostream &printAnonym(ostream &os) const { return os; }
		virtual bool parseAnonym(const Array<const char *> &) { return true; }
		virtual bool canParseAnonym() const { return false; }

		virtual bool validate() const { return true; }

		void share(OptGrp *other);
};

// generic option
class Opt {
	public:
		Opt(OptGrp *aGrp, const char *aName, const char *aDescr);
		virtual ~Opt();

		void cmdLine(CmdLine *aCmdLine) { theCmdLine = aCmdLine; }

		virtual bool parse(const char *name, const char *val) = 0;
		virtual void report(ostream &) const = 0;
		virtual bool visible() const { return true; } // use in reports

		const char *name() const { return theName; }
		const char *type() const { return theType; }
		const char *descr() const { return theDescr; }

	protected:
		OptGrp *theGrp;
		CmdLine *theCmdLine;
		String theName;
		String theType;
		String theDescr;
};


// command line parser
class CmdLine {
	public:
		CmdLine();
		virtual ~CmdLine();

		void configure(const Array<OptGrp*> &groups);
		bool parse(int argc, char *argv[]);

		void usage(ostream &os) const;
		void report(ostream &os) const;

	protected:
		// parse one option at a time
		bool parse(const char *name, const char *val);

	protected:
		Array<char *> theArgs;
		Array<Opt *> theOpts;
		OptGrp *theAnonymParser; // group to use for anonymous options

		String thePrgName;   // program name (argv[0])
};

#endif
