
/*
 * $Id: PortMgr.h,v 1.4 1999/09/07 17:47:15 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_PORT_MANAGER_H
#define PG_PORT_MANAGER_H

#include "LevelStat.h"
#include "BitMask.h"

class Socket;

// manages allocation of network ports
// see EphPortMgr and ExpPortMgr for concrete classes
class PortMgr {
	public:
		PortMgr();
		virtual ~PortMgr() {}

		const LevelStat &boundLvl() const { return theBoundLvl; }

		int bind(Socket &s); // returns port number
		void release(int port, bool good);

		// notify about ports allocated outside of the algorithm
		void reserve(int port) { theResPorts.beSet(port); }
		void cancel(int port) { theResPorts.beClear(port); }

	protected:
		virtual int allocPort(Socket &s) = 0;
		virtual void freePort(int port, bool good) = 0;

	protected:
		BitMask theResPorts;   // reserved ports
		LevelStat theBoundLvl; // bounded ports level
};

#endif
