
/*
 * $Id: TmlModel.h,v 1.4 1999/09/07 17:47:23 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef TMP_LOCALITY_MODEL_H
#define TMP_LOCALITY_MODEL_H

#include "Future.h"

class RndDistr;

// temporal locality model
class TmlModel {
	public:
		TmlModel();

		void distDistr(RndDistr *aDistDistr) { theDistDistr = aDistDistr; }
		void searchDelta(int aDelta) { theSearchDelta = aDelta; }
		void planDepth(int maxDepth) { theFuture.capacity(maxDepth); }

		bool enabled() const { return theDistDistr; }
		const RndDistr *distr() const { return theDistDistr; }
		int planDepth() const { return theFuture.capacity(); }
		int curPlanDepth() const; // careful: may scan the whole plan

		int pickOid();
		bool addOid(int oid);
		void advance() { theFuture.shift(0); }

	protected:
		RndDistr *theDistDistr; // distances between requests
		int theSearchDelta;     // fuzzy selection: [best-d, best+d]

		Future<int> theFuture;
};

#endif
