
/*
 * $Id: World.h,v 1.4 1999/09/07 17:47:26 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef PG_WORLD_CLASS_H
#define PG_WORLD_CLASS_H

#include "Time.h"
#include "Size.h"
#include "String.h"
#include "Array.h"
#include "Oid.h"

class ostream;

class PopModel;
class TmlModel;
class HitModel;

class World {
	public:
		World();
		~World();

		const String &type() const { return theType; }
		const String &id() const { return theId; }
		const char *url(int oid) const { return (oid > 0 && oid <= theUrls.count()) ? theUrls[oid-1] : 0; }
		int oid(int offset, OidRec &rec) { return theObjIds.count() ? theObjIds[offset % theObjIds.count()] : genOid(rec); }
		int uid(int offset) const { return theUnqIds.count() ? theUnqIds[offset % theUnqIds.count()] : offset+1; }

		int genOid(OidRec &rec);
		void noteResponse(const OidRec &) {} // looks like we do not care

		void type(const String &aType) { theType = aType; }
		void id(const String &anId) { theId = anId; }

		void prefill(int count);
		bool loadUrls(const char *fname);

		ostream &report(ostream &os) const;

	protected:
		int repeatOid(int lastId);

	public:
		PopModel *thePopMdl;
		TmlModel *theTmlMdl;
		HitModel *theHitMdl;

	protected:
		String theType;      // universe id, sort of
		String theId;        // world id (to generate unique worlds)

		Array<char*> theUrls;// pre-loaded urls if any
		Array<int> theObjIds;// pre-generated object IDs if any
		Array<int> theUnqIds;// pre-generated unique IDs if any

		int theLastId;

		bool replanOid;      // used internally by genOid routines
};

extern World TheWorld;

#endif
