
/*
 * $Id: polyOpts.h,v 1.8 1999/09/07 17:47:33 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef POLYGRAPH_OPTS_CLASSES_H
#define POLYGRAPH_OPTS_CLASSES_H

#include "Opt.h"
#include "ObjLifeCycle.h"
#include "GoalRec.h"

class FileScanner;

// a collection of command line options that are very specific to Poly
// see include/opts.h for more generic ones


// Goal
class GoalOpt: public Opt, public GoalRec {
	public:
		GoalOpt(OptGrp *aGrp, const char *aName, const char *aDescr);

		virtual bool parse(const char *name, const char *val);
		virtual void report(ostream &) const;

	protected:
		GoalRec &theVal;
};


// which system call to use for scanning for ready files
class FileScanOpt: public Opt {
	public:
		FileScanOpt(OptGrp *aGrp, const char *aName, const char *aDescr, FileScanner *def);

		virtual bool parse(const char *name, const char *val);
		virtual void report(ostream &) const;

		FileScanner *val() { return theVal; }
		operator void*() const { return theVal ? (void*)-1 : 0; }

	protected:
		FileScanner *theVal;
};


// generic interface, but poly-specific implementation
class HostTypeOpt: public Opt {
	public:
		HostTypeOpt(OptGrp *aGrp, const char *aName, const char *aDescr);

		virtual bool parse(const char *name, const char *val);
		virtual void report(ostream &) const;
};

// generic interface, but poly-specific implementation
class VersionOpt: public Opt {
	public:
		VersionOpt(OptGrp *aGrp, const char *aName, const char *aDescr);

		virtual bool parse(const char *name, const char *val);
		virtual void report(ostream &) const;
};


#endif
