
/*
 * $Id: RobotSym.cc,v 1.6 1999/09/07 17:47:52 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "pgl.h"

#include "String.h"
#include "TblDistr.h"
#include "PglIntSym.h"
#include "PglNumSym.h"
#include "PglRec.h"
#include "PglStringSym.h"
#include "PglArraySym.h"
#include "PglNetAddrSym.h"
#include "RobotSym.h"


RobotSym::RobotSym(): AgentSym("Robot") {
	// add fields

	theRec->bAdd("addr[]", "origins", 0);
	theRec->bAdd("time", "launch_win", 0);
	theRec->bAdd("rate", "req_rate", 0);
	theRec->bAdd("float", "dhr", 0);
}

RobotSym::RobotSym(const String &aType, PglRec *aRec): AgentSym(aType, aRec) {
}

bool RobotSym::isA(const String &type) const {
	return AgentSym::isA(type) || type == "Robot";
}

SynSym *RobotSym::clone(const String &type) const {
	if (isA(type))
		return new RobotSym(this->type(), theRec->clone());
	return AgentSym::clone(type);
}

bool RobotSym::reqRate(double &rate) const {
	SynSymTblItem *rri = 0;
	Assert(theRec->find("req_rate", rri));
	if (rri->sym())
		rate = ((const NumSym&)rri->sym()->cast("rate")).val();
	return rri->sym();
}

Time RobotSym::launchWin() const {
	return getTime("launch_win");
}

bool RobotSym::origins(Array<NetAddr> &addrs, RndDistr *&selector) const {
	SynSymTblItem *oi = 0;
	Assert(theRec->find("origins", oi));
	if (!oi->sym())
		return false; // undefined

	// build address array and selector
	ArraySym &os = (ArraySym&)oi->sym()->cast("Array");
	selector = os.makeSelector(kind() + "-origins");
	for (int i = 0; i < os.count(); ++i) {
		NetAddr addr = ((const NetAddrSym&)os[i]->cast("addr")).val();
		addrs.append(addr);
	}

	return true;
}
