
/*
 * $Id: polyOpts.cc,v 1.10 1999/09/07 17:46:42 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "polygraph.h"

#include "Select.h"
#include "Poll.h"
#include "rndDistrs.h"
#include "opts.h"
#include "polyOpts.h"
#include "polyVersion.h"
#include "gadgets.h"


/* Goal */

GoalOpt::GoalOpt(OptGrp *aGrp, const char *aName, const char *aDescr):
	Opt(aGrp, aName, aDescr), theVal(*this) {
}

bool GoalOpt::parse(const char *name, const char *val) {
	if (val) {
		String s[3], tail;
		int count = 1;

		if (splitVal(val, s[0], tail)) {
			count = 2;
			if (splitVal(tail, s[1], s[2]))
				count = 3;
			else
				s[1] = tail;
		} else 
			s[0] = val;

		int dcount = 0;
		int icount = 0;
		bool valid = true;
		// first integer if any is compl count
		// second integer if any is err count
		// time if any is duration
		for (int i = 0; valid && i < count; ++i) {
			const char *ptr;
			if (isTime(s[i], theDuration))
				valid = !dcount++;
			else
			if (!icount++)
				valid = isInt(s[i], theXactCnt, &ptr) && !*ptr;
			else
			if (icount == 2) {
				int h;
				if (isInt(s[i], h,  &ptr) && !*ptr)
					theErrs.theCount = h;
				else
					valid = isRatio(s[i], theErrs.theRatio);
			} else
				valid = false;
		}
		if (valid)
			return true;
	}
	cerr << "goal value (e.g. `1000:1hr:10') expected for the `" << name << "` option; got `" << valStr(val) << "'" << endl;
	return false;
}

void GoalOpt::report(ostream &os) const {
	this->print(os);
}


/* FileScanOpt */

FileScanOpt::FileScanOpt(OptGrp *aGrp, const char *aName, const char *aDescr, FileScanner *def):
	Opt(aGrp, aName, aDescr), theVal(def) {
}

bool FileScanOpt::parse(const char *name, const char *val) {

	delete theVal; theVal = 0;

	const String v = val;
	if (v == "select")
		theVal = new Select();
	else
	if (v == "poll")
		theVal = new Poll();
	else
		cerr << "unknown file scanner name (use `poll' or `select') for the `" << name << "' option; got: " << v << endl;

	return theVal;
}

void FileScanOpt::report(ostream &os) const {
	os << (theVal ? theVal->name() : "<none>");
}


/* HostTypeOpt */

HostTypeOpt::HostTypeOpt(OptGrp *aGrp, const char *aName, const char *aDescr):
	Opt(aGrp, aName, aDescr) {
}

bool HostTypeOpt::parse(const char *, const char *) {
	Assert(theCmdLine);
	cerr
		<< "vesrion " << PolyVersion()
		<< " built on " << CONFIG_HOST_TYPE
		<< " at " << PolyCompTime() << ' ' << PolyCompDate()
		<< endl;
	exit(0);
}

void HostTypeOpt::report(ostream &os) const {
	os << CONFIG_HOST_TYPE;
}


/* VersionOpt */

VersionOpt::VersionOpt(OptGrp *aGrp, const char *aName, const char *aDescr):
	Opt(aGrp, aName, aDescr) {
}

bool VersionOpt::parse(const char *, const char *) {
	Assert(theCmdLine);
	cerr << PolyLetterhd() << endl;
	exit(0);
}

void VersionOpt::report(ostream &os) const {
	os << PolyVersion();
}
