
/*
 * $Id: SynSym.cc,v 1.4 1999/09/07 17:48:17 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "xparser.h"

#include "SynSym.h"


/* SynSym */

SynSym::SynSym(const String &aType): theType(0) {
	type(aType);
}

SynSym::~SynSym() {
	type(0);
}

bool SynSym::isA(const String &type) const {
	return theType && type && !strcmp(type, theType);
}

// may be expensive
bool SynSym::canBe(const String &type) const {
	if (isA(type))
		return true;
	if (SynSym *s = clone(type)) {
		delete s;
		return true;
	}
	return false;
}

SynSym &SynSym::cast(const String &typ) {
	((const SynSym *)this)->cast(typ); // will abort if needed
	return *this;
}

const SynSym &SynSym::cast(const String &typ) const {
	if (!isA(typ))
		cerr << here << "incompatible symbol types: "
			<< type() << " ? " << typ << endl << xabort;
	return *this;
}

SynSym *SynSym::clone(const String &) const {
	return 0;
}

void SynSym::type(const String &aType) {
	theType = aType;
}

ostream &SynSym::print(ostream &os, const String &) const {
	return os << "<something " << type() << "-ish>";
}
