
/*
 * $Id: GLexer.h,v 1.3 1999/09/07 17:48:19 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef XPARSER_GLEXER_H
#define XPARSER_GLEXER_H

#include "Lexer.h"

class istream;
class FinalInfo;

// The GLexer class is the base class from which generated lexer classes
// are derived from.

class GLexer: public Lexer {
	public:
		GLexer(istream &aSource, const String &aFname);
		virtual ~GLexer();

		const String &fname() const { return theFName; }
		istream &source() { return theSource; }

	protected:
		virtual LexToken scan();
		void nextCh();

		virtual int transition(int, int) = 0;
	
	protected:
		istream &theSource;
		String theFName;

		int initialState;
		FinalInfo *finalInfo;

		LexTokenChar _lookahead;
};


// The FinalInfo class provides information about final states of a
// generated FSA.

typedef void (*filterProc)(GLexer *, LexToken &);
struct FinalInfo {
	int tokenValue;
	filterProc filter;
};

#endif
