
/*
 * $Id: Select.cc,v 1.3 1999/09/07 17:48:28 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#include "xstd.h"

#include <unistd.h>

#include "Assert.h"
#include "Select.h"
#include "gadgets.h"


FD_Set::FD_Set(): theMaxFD(-1), theResCount(0) {
	FD_ZERO(&theSet);
	FD_ZERO(&theReadySet);
}

FD_Set::~FD_Set() {
}

void FD_Set::fdLimit(int limit) {
	theUsers.stretch(limit);
}

void FD_Set::setFD(int fd, FileScanUser *u) {
	Assert(fd >= 0 && u);
	Assert(fd < theUsers.capacity());

	Assert(!theUsers[fd]);

	FD_SET(fd, &theSet);
	if (fd > theMaxFD)
		theMaxFD = fd;
	theUsers[fd] = u;
	theResCount++;
}

void FD_Set::clearFD(int fd) {
	Assert(fd >= 0);
	Assert(fd <= theMaxFD);

	Assert(theUsers[fd]);

	FD_CLR(fd, &theSet);
	FD_CLR(fd, &theReadySet); // poll() cannot do that
	if (fd == theMaxFD)
		while (--theMaxFD >= 0 && !FD_ISSET(theMaxFD, &theSet));
	theUsers[fd] = 0;
	theResCount--;
}


/* Select */

Select::Select(): theMaxFD(-1) {
}

void Select::configure(int fdLimit) {
	FileScanner::configure(fdLimit);
	theSets[dirRead].fdLimit(fdLimit);
	theSets[dirWrite].fdLimit(fdLimit);
}

FileScanReserv Select::setFD(int fd, IODir dir, FileScanUser *u) {
	theSets[dir].setFD(fd, u);
	if (fd > theMaxFD)
		theMaxFD = fd;
	return FileScanner::setFD(fd, dir, u);
}

void Select::clearFD(int fd, IODir dir) {
	theSets[dir].clearFD(fd);
	if (fd == theMaxFD)
		theMaxFD = Max(theSets[dirRead].maxFD(), theSets[dirWrite].maxFD());
	FileScanner::clearFD(fd, dir);
}

FileScanUser *Select::readyUser(int idx, IODir dir, int &fd) {
	return theSets[dir].readyUser(fd = idx);
}

int Select::sweep(Time *timeout) {
	// have to check all fds, cannot choose like poll() does
	const int hotCount = theMaxFD+1;

	const int res = ::select(hotCount,
		theSets[dirRead].makeReadySet(),
		theSets[dirWrite].makeReadySet(),
		0,
		timeout);

	return res <= 0 ? res : hotCount;
}
