
/*
 * $Id: NetAddr.h,v 1.6 1999/09/07 17:48:41 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef X_NET_ADDR_H
#define X_NET_ADDR_H

#include <sys/types.h>
#include <netinet/in.h>

// this is needed on Solaris; others?
#ifndef INADDR_NONE
#ifndef _IN_ADDR_T
#define _IN_ADDR_T
typedef unsigned int    in_addr_t;
#endif
#define INADDR_NONE ((in_addr_t)-1)
#endif

#include "String.h"

inline bool operator ==(in_addr a, in_addr b) { return a.s_addr == b.s_addr; }
inline bool operator !=(in_addr a, in_addr b) { return a.s_addr != b.s_addr; }

// inet_addr wrapper which also stores port and
// does _lazy_ ntoa and aton maipulations
class NetAddr {
	public:
		static NetAddr MakeAddr(unsigned long net, unsigned long lna, int port = -1);

	public:
		NetAddr(): thePort(-1) { theAddrN.s_addr = INADDR_NONE; }
		NetAddr(const String &addr, int aPort): theAddrA(addr), thePort(aPort) { theAddrN.s_addr = INADDR_NONE; }
		NetAddr(const in_addr &addr, int aPort): theAddrN(addr), thePort(aPort) {}
		NetAddr(const sockaddr_in &a): theAddrN(a.sin_addr), thePort(a.sin_port) {}

		operator void*() const { return (theAddrA || theAddrN.s_addr != INADDR_NONE || thePort >= 0) ? (void*)-1 : 0; }
		bool operator == (const NetAddr &addr) const;
		bool operator != (const NetAddr &addr) const { return !(*this == addr); }

		const String &addrA() const { if (!theAddrA) syncA(); return theAddrA; }
		in_addr addrN() const { if (theAddrN.s_addr == INADDR_NONE) syncN(); return theAddrN; }
		unsigned long lna() const;
		unsigned long net() const;
		int port() const { return thePort; }

		void addr(const String &addr) { theAddrA = addr; theAddrN.s_addr = INADDR_NONE; }
		void addr(in_addr addr) { theAddrN = addr; theAddrA = 0; }
		void port(int aPort) { thePort = aPort; }

		ostream &print(ostream &os) const;

		//bool fqdn(String &name);

	protected:
		void syncA() const; // theAddrA <- theAddrN
		void syncN() const; // theAddrN <- theAddrA

	protected:
		mutable String theAddrA;  // FQDN or IP; set if not null
		mutable in_addr theAddrN; // IP if available; set if >= 0;
		int thePort;              // set if >= 0;
};

inline ostream &operator <<(ostream &os, const NetAddr &addr) { return addr.print(os); }

#endif
