
/*
 * $Id: Queue.h,v 1.2 1999/09/07 17:48:42 rousskov Exp $
 *
 *
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1999 by Alex Rousskov and the
 *  University of California, San Diego (UCSD); all rights reserved.
 *
 *  You may copy and distribute VERBATIM copies of this product. 
 *  You may modify your copy for your own use. You must NOT
 *  distribute modified copies. You must NOT publish benchmarking
 *  results based on modified versions.
 *
 *  If you publish any materials derived from or related to Web
 *  Polygraph, you must give a credit reference to the IRCache group.
 *
 *  Web Polygraph is distributed without any warranty, even implied.
 *
 *  Please see the COPYRIGHT file for full details.
 *
 */

#ifndef XSTD_QUEUE_H
#define XSTD_QUEUE_H



// general purpose queue with one big limitation:
// a given item can be in no more than one queue at a time

// implements basic queue item actions and holds pointers
class QueueItem {
	public:
		QueueItem(): thePrev(0), theNext(0) {}

		QueueItem *next() const { return theNext; }
		QueueItem *prev() const { return thePrev; }
		bool isolated() const { return !theNext && !thePrev; }

		void next(QueueItem *n) { theNext = n; if (n) n->thePrev = this; }
		void prev(QueueItem *p) { thePrev = p; if (p) p->theNext = this; }
		void isolate() { theNext = thePrev = 0; }

	protected:
		QueueItem *thePrev;
		QueueItem *theNext;
};

// a queue with default FIFO interface
template <class Item>
class Queue {
	public:
		Queue(): theCount(0) { theAncor.next(&theAncor); }

		int count() const { return theCount; }

		inline void enqueueAfter(QueueItem *x, Item *i);
		inline Item *dequeue(Item *i);
		
		// FIFO
		void enqueue(Item *i) { enqueueAfter(theAncor.prev(), i); }
		Item *dequeue() { return dequeue((Item*)theAncor.next()); }

	protected:
		QueueItem theAncor;
		int theCount;
};

/* implementaion of in-lined methods */

template <class Item>
inline
void Queue<Item>::enqueueAfter(QueueItem *x, Item *i) {
	Assert(i->isolated());
	i->next(x->next());
	x->next(i);
	theCount++;
}

template <class Item>
inline
Item *Queue<Item>::dequeue(Item *i) {
	Assert(theCount > 0);
	i->prev()->next(i->next());
	i->isolate();
	theCount--;
	return i;
}

#endif
